#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Burger version data

Package that contains version specific information for burger
"""

## \package burger.__pkginfo__

from __future__ import unicode_literals

## Numeric version
NUMVERSION = (1, 1, 14)

## Current version of the library
VERSION = '.'.join([str(num) for num in NUMVERSION])

## Author's name
AUTHOR = 'Rebecca Ann Heineman'

## Name of the module
TITLE = 'burger'

## Summary of the module's use
SUMMARY = 'Burger Becky\'s shared python library.'

## Home page
URI = 'http://pyburger.readthedocs.io'

## Email address for bug reports
EMAIL = 'becky@burgerbecky.com'

## Type of license used for distribution
LICENSE = 'MIT License'

## Copyright owner
COPYRIGHT = 'Copyright 2013-2018 Rebecca Ann Heineman'
